/**
  * @file SignalRelayAVR.c
  * @brief This module contains the implementation of the signal relay hardware
  * functions.
  *
  * @author Stefan Popovic
  *
  * @date 15/10/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: ATMEGA32
  * Dev.Board: EasyAVR v7 Connectivity
  * External Clock 10.0000 MHz
  * Ext. Modules: Signal Relay Click Board
  * SW: MikroC PRO for AVR v6.1.2
  *
  * NOTES:
    - Signal Relay click board is essentially board with 4 5VDC Signal Relays
    - Place Signal Relay click board on mikroBUS slot 1
    - Put Button Press Level switch PD into upper position and pull-down 
      PORTD3, PORTD2, PORTD1 and PORTD0
    - RE 1: PA7
    - RE 2: PA6
    - RE 3: PA5
    - RE 4: PD4


  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/


  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


   /**************************************************************************
   * Includes
   **************************************************************************/
   #include "built_in.h"
   #include <stdbool.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/

  char txt[24];
  char txt1[] = "RE1";
  char txt2[] = "RE2";
  char txt3[] = "RE3";
  char txt4[] = "RE4";
  
  bool oldD3;
  bool oldD2;
  bool oldD1;
  bool oldD0;

  // LCD module connections
  sbit LCD_RS at PORTA2_bit;
  sbit LCD_EN at PORTD6_bit;
  sbit LCD_D4 at PORTC4_bit;
  sbit LCD_D5 at PORTC5_bit;
  sbit LCD_D6 at PORTC6_bit;
  sbit LCD_D7 at PORTC7_bit;

  sbit LCD_RS_Direction at DDA2_bit;
  sbit LCD_EN_Direction at DDD6_bit;
  sbit LCD_D4_Direction at DDC4_bit;
  sbit LCD_D5_Direction at DDC5_bit;
  sbit LCD_D6_Direction at DDC6_bit;
  sbit LCD_D7_Direction at DDC7_bit;
  // End LCD module connections

   void Display_Init()
  {
    Lcd_Init();                          // Initialize LCD
    Lcd_Cmd(_LCD_CLEAR);                 // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);            // Cursor off
    
     Lcd_Out(1, 1, txt1);
     Lcd_Out(1, 10, txt2);
     Lcd_Out(2, 1, txt3);
     Lcd_Out(2, 10, txt4);
     
     Lcd_Out(1, 5, "OFF");               // RE1
     Lcd_Out(1, 14, "OFF");              // RE2
     Lcd_Out(2, 5, "OFF");               // RE3
     Lcd_Out(2, 14, "OFF");              // RE4
     
   }
   
   void main()
   {
      Display_Init();

      oldD3 = false;
      oldD2 = false;
      oldD1 = false;
      oldD0 = false;
      
      DDA7_bit = 1;                      // Set PORTA pin 7 as output
      DDA6_bit = 1;                      // Set PORTA pin 6 as output
      DDA5_bit = 1;                      // Set PORTA pin 5 as output
      DDD4_bit = 1;                      // Set PORTD pin 4 as output
      
      DDD3_bit = 0;                      // Set PORTD pin 3 as input
      DDD2_bit = 0;                      // Set PORTD pin 2 as input
      DDD1_bit = 0;                      // Set PORTD pin 1 as input
      DDD0_bit = 0;                      // Set PORTD pin 0 as input
      
      PORTA = 0x00;                      // Initial PORTA value
      PORTD = 0x00;                      // Initial PORTD value
      
      while(1) 
      {

          // Relay 1
          if (Button(&PIND, 3, 1, 1))    // detect logical one on PORTD3 pin
            {

              if (!oldD3)                // detect zero-to-one transition
              {
                Delay_ms(200);
                Lcd_Out(1, 5, "    ");
                Lcd_Out(1, 5, "ON");     // RE1
                oldD3 = 1;
                PORTA7_bit = 1;

              }
              else                       // detect one-to-zero transition
              {
                Delay_ms(200);
                Lcd_Out(1, 5, "    ");
                Lcd_Out(1, 5, "OFF");    // RE1
                oldD3 = 0;
                PORTA7_bit = 0;
              }
            }

          // Relay 2
          if (Button(&PIND, 2, 1, 1))    // detect logical one on PORTD2 pin
            {

              if (!oldD2)                // detect zero-to-one transition
              {
                Delay_ms(200);
                Lcd_Out(1, 14, "    ");
                Lcd_Out(1, 14, "ON");    // RE2
                oldD2 = 1;
                PORTA6_bit = 1;

              }
              else                       // detect one-to-zero transition
              {
                Delay_ms(200);
                Lcd_Out(1, 14, "    ");
                Lcd_Out(1, 14, "OFF");   // RE2
                oldD2 = 0;
                PORTA6_bit = 0;
              }
            }

          // Relay 3
          if (Button(&PIND, 1, 1, 1))    // detect logical one on PORTD1 pin
            {

              if (!oldD1)                // detect zero-to-one transition
              {
                Delay_ms(200);
                Lcd_Out(2, 5, "    ");
                Lcd_Out(2, 5, "ON");     // RE3
                oldD1 = 1;
                PORTA5_bit = 1;

               }
              else                       // detect one-to-zero transition
              {
                Delay_ms(200);
                Lcd_Out(2, 5, "     ");
                Lcd_Out(2, 5, "OFF");    // RE3
                oldD1 = 0;
                PORTA5_bit = 0;
              }
            }

          // Relay 4
          if (Button(&PIND, 0, 1, 1))    // detect logical one on PORTD0 pin
            {

              if (!oldD0)                // detect zero-to-one transition
              {
                Delay_ms(200);
                Lcd_Out(2, 14, "    ");
                Lcd_Out(2, 14, "ON");    // RE4
                oldD0 = 1;
                PORTD4_bit = 1;

              }
              else                       // detect one-to-zero transition
              {
                Delay_ms(200);
                Lcd_Out(2, 14, "    ");
                Lcd_Out(2, 14, "OFF");   // RE4
                oldD0 = 0;
                PORTD4_bit = 0;          
              }
            }
      }
   }